#include <unordered_map>
#include "html/Tag.h"

using namespace std;

namespace texty { namespace html {

static const unordered_map<Tag, GumboTag> tagToGumboMap {
  {Tag::HTML, GUMBO_TAG_HTML},
  {Tag::HEAD, GUMBO_TAG_HEAD},
  {Tag::TITLE, GUMBO_TAG_TITLE},
  {Tag::BASE, GUMBO_TAG_BASE},
  {Tag::LINK, GUMBO_TAG_LINK},
  {Tag::META, GUMBO_TAG_META},
  {Tag::STYLE, GUMBO_TAG_STYLE},
  {Tag::SCRIPT, GUMBO_TAG_SCRIPT},
  {Tag::NOSCRIPT, GUMBO_TAG_NOSCRIPT},
  {Tag::TEMPLATE, GUMBO_TAG_TEMPLATE},
  {Tag::BODY, GUMBO_TAG_BODY},
  {Tag::ARTICLE, GUMBO_TAG_ARTICLE},
  {Tag::SECTION, GUMBO_TAG_SECTION},
  {Tag::NAV, GUMBO_TAG_NAV},
  {Tag::ASIDE, GUMBO_TAG_ASIDE},
  {Tag::H1, GUMBO_TAG_H1},
  {Tag::H2, GUMBO_TAG_H2},
  {Tag::H3, GUMBO_TAG_H3},
  {Tag::H4, GUMBO_TAG_H4},
  {Tag::H5, GUMBO_TAG_H5},
  {Tag::H6, GUMBO_TAG_H6},
  {Tag::HGROUP, GUMBO_TAG_HGROUP},
  {Tag::HEADER, GUMBO_TAG_HEADER},
  {Tag::FOOTER, GUMBO_TAG_FOOTER},
  {Tag::ADDRESS, GUMBO_TAG_ADDRESS},
  {Tag::P, GUMBO_TAG_P},
  {Tag::HR, GUMBO_TAG_HR},
  {Tag::PRE, GUMBO_TAG_PRE},
  {Tag::BLOCKQUOTE, GUMBO_TAG_BLOCKQUOTE},
  {Tag::OL, GUMBO_TAG_OL},
  {Tag::UL, GUMBO_TAG_UL},
  {Tag::LI, GUMBO_TAG_LI},
  {Tag::DL, GUMBO_TAG_DL},
  {Tag::DT, GUMBO_TAG_DT},
  {Tag::DD, GUMBO_TAG_DD},
  {Tag::FIGURE, GUMBO_TAG_FIGURE},
  {Tag::FIGCAPTION, GUMBO_TAG_FIGCAPTION},
  {Tag::MAIN, GUMBO_TAG_MAIN},
  {Tag::DIV, GUMBO_TAG_DIV},
  {Tag::A, GUMBO_TAG_A},
  {Tag::EM, GUMBO_TAG_EM},
  {Tag::STRONG, GUMBO_TAG_STRONG},
  {Tag::SMALL, GUMBO_TAG_SMALL},
  {Tag::S, GUMBO_TAG_S},
  {Tag::CITE, GUMBO_TAG_CITE},
  {Tag::Q, GUMBO_TAG_Q},
  {Tag::DFN, GUMBO_TAG_DFN},
  {Tag::ABBR, GUMBO_TAG_ABBR},
  {Tag::DATA, GUMBO_TAG_DATA},
  {Tag::TIME, GUMBO_TAG_TIME},
  {Tag::CODE, GUMBO_TAG_CODE},
  {Tag::VAR, GUMBO_TAG_VAR},
  {Tag::SAMP, GUMBO_TAG_SAMP},
  {Tag::KBD, GUMBO_TAG_KBD},
  {Tag::SUB, GUMBO_TAG_SUB},
  {Tag::SUP, GUMBO_TAG_SUP},
  {Tag::I, GUMBO_TAG_I},
  {Tag::B, GUMBO_TAG_B},
  {Tag::U, GUMBO_TAG_U},
  {Tag::MARK, GUMBO_TAG_MARK},
  {Tag::RUBY, GUMBO_TAG_RUBY},
  {Tag::RT, GUMBO_TAG_RT},
  {Tag::RP, GUMBO_TAG_RP},
  {Tag::BDI, GUMBO_TAG_BDI},
  {Tag::BDO, GUMBO_TAG_BDO},
  {Tag::SPAN, GUMBO_TAG_SPAN},
  {Tag::BR, GUMBO_TAG_BR},
  {Tag::WBR, GUMBO_TAG_WBR},
  {Tag::INS, GUMBO_TAG_INS},
  {Tag::DEL, GUMBO_TAG_DEL},
  {Tag::IMAGE, GUMBO_TAG_IMAGE},
  {Tag::IMG, GUMBO_TAG_IMG},
  {Tag::IFRAME, GUMBO_TAG_IFRAME},
  {Tag::EMBED, GUMBO_TAG_EMBED},
  {Tag::OBJECT, GUMBO_TAG_OBJECT},
  {Tag::PARAM, GUMBO_TAG_PARAM},
  {Tag::VIDEO, GUMBO_TAG_VIDEO},
  {Tag::AUDIO, GUMBO_TAG_AUDIO},
  {Tag::SOURCE, GUMBO_TAG_SOURCE},
  {Tag::TRACK, GUMBO_TAG_TRACK},
  {Tag::CANVAS, GUMBO_TAG_CANVAS},
  {Tag::MAP, GUMBO_TAG_MAP},
  {Tag::AREA, GUMBO_TAG_AREA},
  {Tag::MATH, GUMBO_TAG_MATH},
  {Tag::MI, GUMBO_TAG_MI},
  {Tag::MO, GUMBO_TAG_MO},
  {Tag::MN, GUMBO_TAG_MN},
  {Tag::MS, GUMBO_TAG_MS},
  {Tag::MTEXT, GUMBO_TAG_MTEXT},
  {Tag::MGLYPH, GUMBO_TAG_MGLYPH},
  {Tag::MALIGNMARK, GUMBO_TAG_MALIGNMARK},
  {Tag::ANNOTATION_XML, GUMBO_TAG_ANNOTATION_XML},
  {Tag::SVG, GUMBO_TAG_SVG},
  {Tag::FOREIGNOBJECT, GUMBO_TAG_FOREIGNOBJECT},
  {Tag::DESC, GUMBO_TAG_DESC},
  {Tag::TABLE, GUMBO_TAG_TABLE},
  {Tag::CAPTION, GUMBO_TAG_CAPTION},
  {Tag::COLGROUP, GUMBO_TAG_COLGROUP},
  {Tag::COL, GUMBO_TAG_COL},
  {Tag::TBODY, GUMBO_TAG_TBODY},
  {Tag::THEAD, GUMBO_TAG_THEAD},
  {Tag::TFOOT, GUMBO_TAG_TFOOT},
  {Tag::TR, GUMBO_TAG_TR},
  {Tag::TD, GUMBO_TAG_TD},
  {Tag::TH, GUMBO_TAG_TH},
  {Tag::FORM, GUMBO_TAG_FORM},
  {Tag::FIELDSET, GUMBO_TAG_FIELDSET},
  {Tag::LEGEND, GUMBO_TAG_LEGEND},
  {Tag::LABEL, GUMBO_TAG_LABEL},
  {Tag::INPUT, GUMBO_TAG_INPUT},
  {Tag::BUTTON, GUMBO_TAG_BUTTON},
  {Tag::SELECT, GUMBO_TAG_SELECT},
  {Tag::DATALIST, GUMBO_TAG_DATALIST},
  {Tag::OPTGROUP, GUMBO_TAG_OPTGROUP},
  {Tag::OPTION, GUMBO_TAG_OPTION},
  {Tag::TEXTAREA, GUMBO_TAG_TEXTAREA},
  {Tag::KEYGEN, GUMBO_TAG_KEYGEN},
  {Tag::OUTPUT, GUMBO_TAG_OUTPUT},
  {Tag::PROGRESS, GUMBO_TAG_PROGRESS},
  {Tag::METER, GUMBO_TAG_METER},
  {Tag::DETAILS, GUMBO_TAG_DETAILS},
  {Tag::SUMMARY, GUMBO_TAG_SUMMARY},
  {Tag::MENU, GUMBO_TAG_MENU},
  {Tag::MENUITEM, GUMBO_TAG_MENUITEM},
  {Tag::APPLET, GUMBO_TAG_APPLET},
  {Tag::ACRONYM, GUMBO_TAG_ACRONYM},
  {Tag::BGSOUND, GUMBO_TAG_BGSOUND},
  {Tag::DIR, GUMBO_TAG_DIR},
  {Tag::FRAME, GUMBO_TAG_FRAME},
  {Tag::FRAMESET, GUMBO_TAG_FRAMESET},
  {Tag::NOFRAMES, GUMBO_TAG_NOFRAMES},
  {Tag::ISINDEX, GUMBO_TAG_ISINDEX},
  {Tag::LISTING, GUMBO_TAG_LISTING},
  {Tag::XMP, GUMBO_TAG_XMP},
  {Tag::NEXTID, GUMBO_TAG_NEXTID},
  {Tag::NOEMBED, GUMBO_TAG_NOEMBED},
  {Tag::PLAINTEXT, GUMBO_TAG_PLAINTEXT},
  {Tag::RB, GUMBO_TAG_RB},
  {Tag::STRIKE, GUMBO_TAG_STRIKE},
  {Tag::BASEFONT, GUMBO_TAG_BASEFONT},
  {Tag::BIG, GUMBO_TAG_BIG},
  {Tag::BLINK, GUMBO_TAG_BLINK},
  {Tag::CENTER, GUMBO_TAG_CENTER},
  {Tag::FONT, GUMBO_TAG_FONT},
  {Tag::MARQUEE, GUMBO_TAG_MARQUEE},
  {Tag::MULTICOL, GUMBO_TAG_MULTICOL},
  {Tag::NOBR, GUMBO_TAG_NOBR},
  {Tag::SPACER, GUMBO_TAG_SPACER},
  {Tag::TT, GUMBO_TAG_TT},
  {Tag::RTC, GUMBO_TAG_RTC}
};

static const unordered_map<GumboTag, Tag> gumboToTagMap {
  {GUMBO_TAG_HTML, Tag::HTML},
  {GUMBO_TAG_HEAD, Tag::HEAD},
  {GUMBO_TAG_TITLE, Tag::TITLE},
  {GUMBO_TAG_BASE, Tag::BASE},
  {GUMBO_TAG_LINK, Tag::LINK},
  {GUMBO_TAG_META, Tag::META},
  {GUMBO_TAG_STYLE, Tag::STYLE},
  {GUMBO_TAG_SCRIPT, Tag::SCRIPT},
  {GUMBO_TAG_NOSCRIPT, Tag::NOSCRIPT},
  {GUMBO_TAG_TEMPLATE, Tag::TEMPLATE},
  {GUMBO_TAG_BODY, Tag::BODY},
  {GUMBO_TAG_ARTICLE, Tag::ARTICLE},
  {GUMBO_TAG_SECTION, Tag::SECTION},
  {GUMBO_TAG_NAV, Tag::NAV},
  {GUMBO_TAG_ASIDE, Tag::ASIDE},
  {GUMBO_TAG_H1, Tag::H1},
  {GUMBO_TAG_H2, Tag::H2},
  {GUMBO_TAG_H3, Tag::H3},
  {GUMBO_TAG_H4, Tag::H4},
  {GUMBO_TAG_H5, Tag::H5},
  {GUMBO_TAG_H6, Tag::H6},
  {GUMBO_TAG_HGROUP, Tag::HGROUP},
  {GUMBO_TAG_HEADER, Tag::HEADER},
  {GUMBO_TAG_FOOTER, Tag::FOOTER},
  {GUMBO_TAG_ADDRESS, Tag::ADDRESS},
  {GUMBO_TAG_P, Tag::P},
  {GUMBO_TAG_HR, Tag::HR},
  {GUMBO_TAG_PRE, Tag::PRE},
  {GUMBO_TAG_BLOCKQUOTE, Tag::BLOCKQUOTE},
  {GUMBO_TAG_OL, Tag::OL},
  {GUMBO_TAG_UL, Tag::UL},
  {GUMBO_TAG_LI, Tag::LI},
  {GUMBO_TAG_DL, Tag::DL},
  {GUMBO_TAG_DT, Tag::DT},
  {GUMBO_TAG_DD, Tag::DD},
  {GUMBO_TAG_FIGURE, Tag::FIGURE},
  {GUMBO_TAG_FIGCAPTION, Tag::FIGCAPTION},
  {GUMBO_TAG_MAIN, Tag::MAIN},
  {GUMBO_TAG_DIV, Tag::DIV},
  {GUMBO_TAG_A, Tag::A},
  {GUMBO_TAG_EM, Tag::EM},
  {GUMBO_TAG_STRONG, Tag::STRONG},
  {GUMBO_TAG_SMALL, Tag::SMALL},
  {GUMBO_TAG_S, Tag::S},
  {GUMBO_TAG_CITE, Tag::CITE},
  {GUMBO_TAG_Q, Tag::Q},
  {GUMBO_TAG_DFN, Tag::DFN},
  {GUMBO_TAG_ABBR, Tag::ABBR},
  {GUMBO_TAG_DATA, Tag::DATA},
  {GUMBO_TAG_TIME, Tag::TIME},
  {GUMBO_TAG_CODE, Tag::CODE},
  {GUMBO_TAG_VAR, Tag::VAR},
  {GUMBO_TAG_SAMP, Tag::SAMP},
  {GUMBO_TAG_KBD, Tag::KBD},
  {GUMBO_TAG_SUB, Tag::SUB},
  {GUMBO_TAG_SUP, Tag::SUP},
  {GUMBO_TAG_I, Tag::I},
  {GUMBO_TAG_B, Tag::B},
  {GUMBO_TAG_U, Tag::U},
  {GUMBO_TAG_MARK, Tag::MARK},
  {GUMBO_TAG_RUBY, Tag::RUBY},
  {GUMBO_TAG_RT, Tag::RT},
  {GUMBO_TAG_RP, Tag::RP},
  {GUMBO_TAG_BDI, Tag::BDI},
  {GUMBO_TAG_BDO, Tag::BDO},
  {GUMBO_TAG_SPAN, Tag::SPAN},
  {GUMBO_TAG_BR, Tag::BR},
  {GUMBO_TAG_WBR, Tag::WBR},
  {GUMBO_TAG_INS, Tag::INS},
  {GUMBO_TAG_DEL, Tag::DEL},
  {GUMBO_TAG_IMAGE, Tag::IMAGE},
  {GUMBO_TAG_IMG, Tag::IMG},
  {GUMBO_TAG_IFRAME, Tag::IFRAME},
  {GUMBO_TAG_EMBED, Tag::EMBED},
  {GUMBO_TAG_OBJECT, Tag::OBJECT},
  {GUMBO_TAG_PARAM, Tag::PARAM},
  {GUMBO_TAG_VIDEO, Tag::VIDEO},
  {GUMBO_TAG_AUDIO, Tag::AUDIO},
  {GUMBO_TAG_SOURCE, Tag::SOURCE},
  {GUMBO_TAG_TRACK, Tag::TRACK},
  {GUMBO_TAG_CANVAS, Tag::CANVAS},
  {GUMBO_TAG_MAP, Tag::MAP},
  {GUMBO_TAG_AREA, Tag::AREA},
  {GUMBO_TAG_MATH, Tag::MATH},
  {GUMBO_TAG_MI, Tag::MI},
  {GUMBO_TAG_MO, Tag::MO},
  {GUMBO_TAG_MN, Tag::MN},
  {GUMBO_TAG_MS, Tag::MS},
  {GUMBO_TAG_MTEXT, Tag::MTEXT},
  {GUMBO_TAG_MGLYPH, Tag::MGLYPH},
  {GUMBO_TAG_MALIGNMARK, Tag::MALIGNMARK},
  {GUMBO_TAG_ANNOTATION_XML, Tag::ANNOTATION_XML},
  {GUMBO_TAG_SVG, Tag::SVG},
  {GUMBO_TAG_FOREIGNOBJECT, Tag::FOREIGNOBJECT},
  {GUMBO_TAG_DESC, Tag::DESC},
  {GUMBO_TAG_TABLE, Tag::TABLE},
  {GUMBO_TAG_CAPTION, Tag::CAPTION},
  {GUMBO_TAG_COLGROUP, Tag::COLGROUP},
  {GUMBO_TAG_COL, Tag::COL},
  {GUMBO_TAG_TBODY, Tag::TBODY},
  {GUMBO_TAG_THEAD, Tag::THEAD},
  {GUMBO_TAG_TFOOT, Tag::TFOOT},
  {GUMBO_TAG_TR, Tag::TR},
  {GUMBO_TAG_TD, Tag::TD},
  {GUMBO_TAG_TH, Tag::TH},
  {GUMBO_TAG_FORM, Tag::FORM},
  {GUMBO_TAG_FIELDSET, Tag::FIELDSET},
  {GUMBO_TAG_LEGEND, Tag::LEGEND},
  {GUMBO_TAG_LABEL, Tag::LABEL},
  {GUMBO_TAG_INPUT, Tag::INPUT},
  {GUMBO_TAG_BUTTON, Tag::BUTTON},
  {GUMBO_TAG_SELECT, Tag::SELECT},
  {GUMBO_TAG_DATALIST, Tag::DATALIST},
  {GUMBO_TAG_OPTGROUP, Tag::OPTGROUP},
  {GUMBO_TAG_OPTION, Tag::OPTION},
  {GUMBO_TAG_TEXTAREA, Tag::TEXTAREA},
  {GUMBO_TAG_KEYGEN, Tag::KEYGEN},
  {GUMBO_TAG_OUTPUT, Tag::OUTPUT},
  {GUMBO_TAG_PROGRESS, Tag::PROGRESS},
  {GUMBO_TAG_METER, Tag::METER},
  {GUMBO_TAG_DETAILS, Tag::DETAILS},
  {GUMBO_TAG_SUMMARY, Tag::SUMMARY},
  {GUMBO_TAG_MENU, Tag::MENU},
  {GUMBO_TAG_MENUITEM, Tag::MENUITEM},
  {GUMBO_TAG_APPLET, Tag::APPLET},
  {GUMBO_TAG_ACRONYM, Tag::ACRONYM},
  {GUMBO_TAG_BGSOUND, Tag::BGSOUND},
  {GUMBO_TAG_DIR, Tag::DIR},
  {GUMBO_TAG_FRAME, Tag::FRAME},
  {GUMBO_TAG_FRAMESET, Tag::FRAMESET},
  {GUMBO_TAG_NOFRAMES, Tag::NOFRAMES},
  {GUMBO_TAG_ISINDEX, Tag::ISINDEX},
  {GUMBO_TAG_LISTING, Tag::LISTING},
  {GUMBO_TAG_XMP, Tag::XMP},
  {GUMBO_TAG_NEXTID, Tag::NEXTID},
  {GUMBO_TAG_NOEMBED, Tag::NOEMBED},
  {GUMBO_TAG_PLAINTEXT, Tag::PLAINTEXT},
  {GUMBO_TAG_RB, Tag::RB},
  {GUMBO_TAG_STRIKE, Tag::STRIKE},
  {GUMBO_TAG_BASEFONT, Tag::BASEFONT},
  {GUMBO_TAG_BIG, Tag::BIG},
  {GUMBO_TAG_BLINK, Tag::BLINK},
  {GUMBO_TAG_CENTER, Tag::CENTER},
  {GUMBO_TAG_FONT, Tag::FONT},
  {GUMBO_TAG_MARQUEE, Tag::MARQUEE},
  {GUMBO_TAG_MULTICOL, Tag::MULTICOL},
  {GUMBO_TAG_NOBR, Tag::NOBR},
  {GUMBO_TAG_SPACER, Tag::SPACER},
  {GUMBO_TAG_TT, Tag::TT},
  {GUMBO_TAG_RTC, Tag::RTC}
};

static const unordered_map<Tag, string> tagToStringMap {
  {Tag::HTML, "HTML"},
  {Tag::HEAD, "HEAD"},
  {Tag::TITLE, "TITLE"},
  {Tag::BASE, "BASE"},
  {Tag::LINK, "LINK"},
  {Tag::META, "META"},
  {Tag::STYLE, "STYLE"},
  {Tag::SCRIPT, "SCRIPT"},
  {Tag::NOSCRIPT, "NOSCRIPT"},
  {Tag::TEMPLATE, "TEMPLATE"},
  {Tag::BODY, "BODY"},
  {Tag::ARTICLE, "ARTICLE"},
  {Tag::SECTION, "SECTION"},
  {Tag::NAV, "NAV"},
  {Tag::ASIDE, "ASIDE"},
  {Tag::H1, "H1"},
  {Tag::H2, "H2"},
  {Tag::H3, "H3"},
  {Tag::H4, "H4"},
  {Tag::H5, "H5"},
  {Tag::H6, "H6"},
  {Tag::HGROUP, "HGROUP"},
  {Tag::HEADER, "HEADER"},
  {Tag::FOOTER, "FOOTER"},
  {Tag::ADDRESS, "ADDRESS"},
  {Tag::P, "P"},
  {Tag::HR, "HR"},
  {Tag::PRE, "PRE"},
  {Tag::BLOCKQUOTE, "BLOCKQUOTE"},
  {Tag::OL, "OL"},
  {Tag::UL, "UL"},
  {Tag::LI, "LI"},
  {Tag::DL, "DL"},
  {Tag::DT, "DT"},
  {Tag::DD, "DD"},
  {Tag::FIGURE, "FIGURE"},
  {Tag::FIGCAPTION, "FIGCAPTION"},
  {Tag::MAIN, "MAIN"},
  {Tag::DIV, "DIV"},
  {Tag::A, "A"},
  {Tag::EM, "EM"},
  {Tag::STRONG, "STRONG"},
  {Tag::SMALL, "SMALL"},
  {Tag::S, "S"},
  {Tag::CITE, "CITE"},
  {Tag::Q, "Q"},
  {Tag::DFN, "DFN"},
  {Tag::ABBR, "ABBR"},
  {Tag::DATA, "DATA"},
  {Tag::TIME, "TIME"},
  {Tag::CODE, "CODE"},
  {Tag::VAR, "VAR"},
  {Tag::SAMP, "SAMP"},
  {Tag::KBD, "KBD"},
  {Tag::SUB, "SUB"},
  {Tag::SUP, "SUP"},
  {Tag::I, "I"},
  {Tag::B, "B"},
  {Tag::U, "U"},
  {Tag::MARK, "MARK"},
  {Tag::RUBY, "RUBY"},
  {Tag::RT, "RT"},
  {Tag::RP, "RP"},
  {Tag::BDI, "BDI"},
  {Tag::BDO, "BDO"},
  {Tag::SPAN, "SPAN"},
  {Tag::BR, "BR"},
  {Tag::WBR, "WBR"},
  {Tag::INS, "INS"},
  {Tag::DEL, "DEL"},
  {Tag::IMAGE, "IMAGE"},
  {Tag::IMG, "IMG"},
  {Tag::IFRAME, "IFRAME"},
  {Tag::EMBED, "EMBED"},
  {Tag::OBJECT, "OBJECT"},
  {Tag::PARAM, "PARAM"},
  {Tag::VIDEO, "VIDEO"},
  {Tag::AUDIO, "AUDIO"},
  {Tag::SOURCE, "SOURCE"},
  {Tag::TRACK, "TRACK"},
  {Tag::CANVAS, "CANVAS"},
  {Tag::MAP, "MAP"},
  {Tag::AREA, "AREA"},
  {Tag::MATH, "MATH"},
  {Tag::MI, "MI"},
  {Tag::MO, "MO"},
  {Tag::MN, "MN"},
  {Tag::MS, "MS"},
  {Tag::MTEXT, "MTEXT"},
  {Tag::MGLYPH, "MGLYPH"},
  {Tag::MALIGNMARK, "MALIGNMARK"},
  {Tag::ANNOTATION_XML, "ANNOTATION_XML"},
  {Tag::SVG, "SVG"},
  {Tag::FOREIGNOBJECT, "FOREIGNOBJECT"},
  {Tag::DESC, "DESC"},
  {Tag::TABLE, "TABLE"},
  {Tag::CAPTION, "CAPTION"},
  {Tag::COLGROUP, "COLGROUP"},
  {Tag::COL, "COL"},
  {Tag::TBODY, "TBODY"},
  {Tag::THEAD, "THEAD"},
  {Tag::TFOOT, "TFOOT"},
  {Tag::TR, "TR"},
  {Tag::TD, "TD"},
  {Tag::TH, "TH"},
  {Tag::FORM, "FORM"},
  {Tag::FIELDSET, "FIELDSET"},
  {Tag::LEGEND, "LEGEND"},
  {Tag::LABEL, "LABEL"},
  {Tag::INPUT, "INPUT"},
  {Tag::BUTTON, "BUTTON"},
  {Tag::SELECT, "SELECT"},
  {Tag::DATALIST, "DATALIST"},
  {Tag::OPTGROUP, "OPTGROUP"},
  {Tag::OPTION, "OPTION"},
  {Tag::TEXTAREA, "TEXTAREA"},
  {Tag::KEYGEN, "KEYGEN"},
  {Tag::OUTPUT, "OUTPUT"},
  {Tag::PROGRESS, "PROGRESS"},
  {Tag::METER, "METER"},
  {Tag::DETAILS, "DETAILS"},
  {Tag::SUMMARY, "SUMMARY"},
  {Tag::MENU, "MENU"},
  {Tag::MENUITEM, "MENUITEM"},
  {Tag::APPLET, "APPLET"},
  {Tag::ACRONYM, "ACRONYM"},
  {Tag::BGSOUND, "BGSOUND"},
  {Tag::DIR, "DIR"},
  {Tag::FRAME, "FRAME"},
  {Tag::FRAMESET, "FRAMESET"},
  {Tag::NOFRAMES, "NOFRAMES"},
  {Tag::ISINDEX, "ISINDEX"},
  {Tag::LISTING, "LISTING"},
  {Tag::XMP, "XMP"},
  {Tag::NEXTID, "NEXTID"},
  {Tag::NOEMBED, "NOEMBED"},
  {Tag::PLAINTEXT, "PLAINTEXT"},
  {Tag::RB, "RB"},
  {Tag::STRIKE, "STRIKE"},
  {Tag::BASEFONT, "BASEFONT"},
  {Tag::BIG, "BIG"},
  {Tag::BLINK, "BLINK"},
  {Tag::CENTER, "CENTER"},
  {Tag::FONT, "FONT"},
  {Tag::MARQUEE, "MARQUEE"},
  {Tag::MULTICOL, "MULTICOL"},
  {Tag::NOBR, "NOBR"},
  {Tag::SPACER, "SPACER"},
  {Tag::TT, "TT"},
  {Tag::RTC, "RTC"}  
};

static const string unknownStr = "UNKNOWN";

Tag tagOfGumboTag(GumboTag gumboTag) {
  auto found = gumboToTagMap.find(gumboTag);
  if (found != gumboToTagMap.end()) {
    return found->second;
  }
  return Tag::UNKNOWN;
}

GumboTag gumboTagOfTag(Tag tag) {
  auto found = tagToGumboMap.find(tag);
  if (found != tagToGumboMap.end()) {
    return found->second;
  }
  return GUMBO_TAG_UNKNOWN;
}

const std::string& stringOfTag(Tag tag) {
  auto found = tagToStringMap.find(tag);
  if (found != tagToStringMap.end()) {
    return found->second;
  }
  return unknownStr;
}

}} // texty::html