#pragma once
#include <gumbo.h>
#include <string>
#include "util/misc.h"

namespace texty { namespace html {

enum class Tag {
  HTML,
  HEAD,
  TITLE,
  BASE,
  LINK,
  META,
  STYLE,
  SCRIPT,
  NOSCRIPT,
  TEMPLATE,
  BODY,
  ARTICLE,
  SECTION,
  NAV,
  ASIDE,
  H1,
  H2,
  H3,
  H4,
  H5,
  H6,
  HGROUP,
  HEADER,
  FOOTER,
  ADDRESS,
  P,
  HR,
  PRE,
  BLOCKQUOTE,
  OL,
  UL,
  LI,
  DL,
  DT,
  DD,
  FIGURE,
  FIGCAPTION,
  MAIN,
  DIV,
  A,
  EM,
  STRONG,
  SMALL,
  S,
  CITE,
  Q,
  DFN,
  ABBR,
  DATA,
  TIME,
  CODE,
  VAR,
  SAMP,
  KBD,
  SUB,
  SUP,
  I,
  B,
  U,
  MARK,
  RUBY,
  RT,
  RP,
  BDI,
  BDO,
  SPAN,
  BR,
  WBR,
  INS,
  DEL,
  IMAGE,
  IMG,
  IFRAME,
  EMBED,
  OBJECT,
  PARAM,
  VIDEO,
  AUDIO,
  SOURCE,
  TRACK,
  CANVAS,
  MAP,
  AREA,
  MATH,
  MI,
  MO,
  MN,
  MS,
  MTEXT,
  MGLYPH,
  MALIGNMARK,
  ANNOTATION_XML,
  SVG,
  FOREIGNOBJECT,
  DESC,
  TABLE,
  CAPTION,
  COLGROUP,
  COL,
  TBODY,
  THEAD,
  TFOOT,
  TR,
  TD,
  TH,
  FORM,
  FIELDSET,
  LEGEND,
  LABEL,
  INPUT,
  BUTTON,
  SELECT,
  DATALIST,
  OPTGROUP,
  OPTION,
  TEXTAREA,
  KEYGEN,
  OUTPUT,
  PROGRESS,
  METER,
  DETAILS,
  SUMMARY,
  MENU,
  MENUITEM,
  APPLET,
  ACRONYM,
  BGSOUND,
  DIR,
  FRAME,
  FRAMESET,
  NOFRAMES,
  ISINDEX,
  LISTING,
  XMP,
  NEXTID,
  NOEMBED,
  PLAINTEXT,
  RB,
  STRIKE,
  BASEFONT,
  BIG,
  BLINK,
  CENTER,
  FONT,
  MARQUEE,
  MULTICOL,
  NOBR,
  SPACER,
  TT,
  RTC,
  UNKNOWN
};

Tag tagOfGumboTag(GumboTag);
GumboTag gumboTagOfTag(Tag);
const std::string& stringOfTag(Tag);

}};

namespace std {
  template<>
  struct hash<texty::html::Tag> {
    using tag_type = texty::html::Tag;
    size_t operator()(tag_type tag) const {
      return texty::util::hashEnum(tag);
    }
  };

  template<>
  struct hash<GumboTag> {
    using tag_type = GumboTag;
    size_t operator()(tag_type tag) const {
      return texty::util::hashEnum(tag);
    }
  };  
}
